@@ -508,11 +508,10 @@ find_identical_ends (filevec)
       beg0 = filevec[0].prefix_end + (n0 < n1 ? 0 : n0 - n1);
 
       /* Scan back until chars don't match or we reach that point.  */
-      while (p0 != beg0)
-	if (*--p0 != *--p1)
+      for (; p0 != beg0; p0--, p1--)
+	if (*p0 != *p1)
 	  {
 	    /* Point at the first char of the matching suffix.  */
-	    ++p0, ++p1;
 	    beg0 = p0;
 	    break;
 	  }
