@@ -3,7 +3,8 @@
  *    Copyright (c) 1989-1992, Brian Berliner
  *
  *    You may distribute under the terms of the GNU General Public License
- *    as specified in the README file that comes with the CVS source distribution.
+ *    as specified in the README file that comes with the CVS source
+ *    distribution.
  *
  * Modules
  *
@@ -91,14 +92,17 @@ close_module (db)
 	dbm_close (db);
 }
 
+
+
 /*
  * This is the recursive function that processes a module name.
  * It calls back the passed routine for each directory of a module
  * It runs the post checkout or post tag proc from the modules file
  */
-int
-do_module (db, mname, m_type, msg, callback_proc, where, shorten,
-	   local_specified, run_module_prog, build_dirs, extra_arg)
+static int
+my_module (db, mname, m_type, msg, callback_proc, where, shorten,
+	   local_specified, run_module_prog, build_dirs, extra_arg,
+	   stack)
     DBM *db;
     char *mname;
     enum mtype m_type;
@@ -110,6 +114,7 @@ do_module (db, mname, m_type, msg, callback_proc, where, shorten,
     int run_module_prog;
     int build_dirs;
     char *extra_arg;
+    List *stack;
 {
     char *checkout_prog = NULL;
     char *export_prog = NULL;
@@ -148,7 +153,7 @@ do_module (db, mname, m_type, msg, callback_proc, where, shorten,
 		       + strlen (msg)
 		       + (where ? strlen (where) : 0)
 		       + (extra_arg ? strlen (extra_arg) : 0));
-	sprintf (buf, "%s-> do_module (%s, %s, %s, %s)\n",
+	sprintf (buf, "%s-> my_module (%s, %s, %s, %s)\n",
 		 CLIENT_SERVER_STR,
 		 mname, msg, where ? where : "",
 		 extra_arg ? extra_arg : "");
@@ -157,6 +162,24 @@ do_module (db, mname, m_type, msg, callback_proc, where, shorten,
     }
 #endif
 
+    /* Don't process absolute directories.  Anything else could be a security
+     * problem.  Before this check was put in place:
+     *
+     *   $ cvs -d:fork:/cvsroot co /foo
+     *   cvs server: warning: cannot make directory CVS in /: Permission denied
+     *   cvs [server aborted]: cannot make directory /foo: Permission denied
+     *   $
+     */
+    if (isabsolute (mname))
+	error (1, 0, "Absolute module reference invalid: `%s'", mname);
+
+    /* Similarly for directories that attempt to step above the root of the
+     * repository.
+     */
+    if (pathname_levels (mname) > 0)
+	error (1, 0, "up-level in module reference (`..') invalid: `%s'.",
+               mname);
+
     /* if this is a directory to ignore, add it to that list */
     if (mname[0] == '!' && mname[1] != '\0')
     {
@@ -225,7 +248,8 @@ do_module (db, mname, m_type, msg, callback_proc, where, shorten,
 	    *acp = '/';
 	}
 	else
-	    (void) sprintf (attic_file, "%s/%s/%s%s", current_parsed_root->directory,
+	    (void) sprintf (attic_file, "%s/%s/%s%s",
+	                    current_parsed_root->directory,
 			    CVSATTIC, mname, RCSEXT);
 
 	if (isdir (file))
@@ -485,14 +509,33 @@ do_module (db, mname, m_type, msg, callback_proc, where, shorten,
 
 	for (i = 0; i < modargc; i++)
 	{
-	    if (strcmp (mname, modargv[i]) == 0)
+	    /* 
+	     * Recursion check: if an alias module calls itself or a module
+	     * which causes the first to be called again, print an error
+	     * message and stop recursing.
+	     *
+	     * Algorithm:
+	     *
+	     *   1. Check that MNAME isn't in the stack.
+	     *   2. Push MNAME onto the stack.
+	     *   3. Call do_module().
+	     *   4. Pop MNAME from the stack.
+	     */
+	    if (stack && findnode (stack, mname))
 		error (0, 0,
 		       "module `%s' in modules file contains infinite loop",
 		       mname);
 	    else
-		err += do_module (db, modargv[i], m_type, msg, callback_proc,
-				  where, shorten, local_specified,
-				  run_module_prog, build_dirs, extra_arg);
+	    {
+		if (!stack) stack = getlist();
+		push_string (stack, mname);
+		err += my_module (db, modargv[i], m_type, msg, callback_proc,
+                                   where, shorten, local_specified,
+                                   run_module_prog, build_dirs, extra_arg,
+                                   stack);
+		pop_string (stack);
+		if (isempty (stack)) dellist (&stack);
+	    }
 	}
 	goto do_module_return;
     }
@@ -527,7 +570,7 @@ module `%s' is a request for a file in a module which is not a directory",
 	/* XXX - think about making null repositories at each dir here
 		 instead of just at the bottom */
 	make_directories (dir);
-	if ( CVS_CHDIR (dir) < 0)
+	if (CVS_CHDIR (dir) < 0)
 	{
 	    error (0, errno, "cannot chdir to %s", dir);
 	    spec_opt = NULL;
@@ -629,9 +672,10 @@ module `%s' is a request for a file in a module which is not a directory",
 	    error (0, 0, "Mal-formed %c option for module %s - ignored",
 		   CVSMODULE_SPEC, mname);
 	else
-	    err += do_module (db, spec_opt, m_type, msg, callback_proc,
-			      (char *) NULL, 0, local_specified,
-			      run_module_prog, build_dirs, extra_arg);
+	    err += my_module (db, spec_opt, m_type, msg, callback_proc,
+                               (char *) NULL, 0, local_specified,
+                               run_module_prog, build_dirs, extra_arg,
+	                       stack);
 	spec_opt = next_opt;
     }
 
@@ -691,7 +735,7 @@ module `%s' is a request for a file in a module which is not a directory",
 		{
 		    cvs_output (program_name, 0);
 		    cvs_output (" ", 1);
-		    cvs_output (command_name, 0);
+		    cvs_output (cvs_cmd_name, 0);
 		    cvs_output (": Executing '", 0);
 		    run_print (stdout);
 		    cvs_output ("'\n", 0);
@@ -726,6 +770,33 @@ module `%s' is a request for a file in a module which is not a directory",
     return (err);
 }
 
+
+
+/* External face of do_module so that we can have an internal version which
+ * accepts a stack argument to track alias recursion.
+ */
+int
+do_module (db, mname, m_type, msg, callback_proc, where, shorten,
+	   local_specified, run_module_prog, build_dirs, extra_arg)
+    DBM *db;
+    char *mname;
+    enum mtype m_type;
+    char *msg;
+    CALLBACKPROC callback_proc;
+    char *where;
+    int shorten;
+    int local_specified;
+    int run_module_prog;
+    int build_dirs;
+    char *extra_arg;
+{
+    return my_module (db, mname, m_type, msg, callback_proc, where, shorten,
+                       local_specified, run_module_prog, build_dirs, extra_arg,
+                       NULL);
+}
+
+
+
 /* - Read all the records from the modules database into an array.
    - Sort the array depending on what format is desired.
    - Print the array in the format desired.
